<?php

/**
 * @file
 * xc_search_property_group is a top level search interface definition element.
 * Now it connects facet group, and highlighter settings, later it will contain
 * a form definition link. The base concept here is that each search UI will be 
 * defined by these elements.
 * Field list: sui_id, name, facet_id (->xc_search_facet_group), 
 * highlighter_id (->xc_search_highlighter), do_prepopulate_facets
 */

/**
 * Get title
 * @param $record (Object)
 *   The search UI object
 * @return (String)
 *   The name of search UI as it will be the title
 */
function xc_search_ui_title($record) {
  return $record->name;
}

/**
 * Load the record from the DB according to its ID
 * @param $sui_id (int)
 *   The search UI identifier
 * @return (Object)
 *   The search UI object
 */
function xc_search_ui_load($sui_id) {
  $sql = "SELECT * FROM {xc_search_ui} WHERE sui_id = %d";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_ui}', 'sui_id'), $sui_id);
  $data = db_fetch_object($result);
  return $data;
} 

/**
 * Load the search ui with the given UI ID, or the first one.
 * @param $sui_id
 * @return unknown_type
 */
function xc_search_ui_get($sui_id = NULL) {
  if(!is_null($sui_id)) {
    return xc_search_ui_load($sui_id);
  }
  // TODO_ later we should create a radio button based selector to select the
  // default one, and not use the first one
  $sql = "SELECT * FROM {xc_search_ui} ORDER BY sui_id LIMIT 1";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_ui}', 'sui_id'));
  $data = db_fetch_object($result);
  return $data;
} 

/**
 * Return the field name -> human readable label pairs of the fields
 * @return (Array)
 */
function xc_search_ui_fieldset() {
  return array(
    'name' => t('Name'),
    'facet_id' => t('Facet group'),
    'highlighter_id' => t('Highlighter'),
    'do_prepopulate_facets' => t('Prepopulate facets?')
  );
}

/**
 * Get the enumerated field values
 * @return (Array)
 */
function xc_search_ui_options() {
  global $XC_SEARCH_GLOBAL_OPTIONS;
  return array(
    'facet_id' => xc_search_facet_group_list_values(),
    'highlighter_id' => xc_search_highlighter_list_values(),
    'do_prepopulate_facets' => $XC_SEARCH_GLOBAL_OPTIONS['true_false'],
  );
}

/**
 * Show the full display of the record
 * @param $record (Object)
 *   The search UI object
 * @return (String)
 *   The rendered view of search UI record
 */
function xc_search_ui_view($record) {
  return xc_util_view($record, 
    xc_search_ui_fieldset(),
    xc_search_ui_options()
  );
}

/**
 * List all search UIs
 * @return (String)
 *   The rendered list of search UIs
 */
function xc_search_ui_list() {
  $headers = array(t('Name'), t('Facet group'), t('Highlighter'), t('edit'), 
    t('delete'));
  $rows = array();
  $sql = 'SELECT ui.*, grp.name AS facet, hl.name AS hlt'
       . ' FROM {xc_search_ui} AS ui'
       . ' LEFT JOIN {xc_search_facet_group} AS grp ON '
       . '   grp.facet_id = ui.facet_id'
       . ' LEFT JOIN {xc_search_highlighter} AS hl ON '
       . '   hl.highlighter_id = ui.highlighter_id';
  $result = db_query(db_rewrite_sql($sql, '{xc_search_ui}', 'sui_id'));
  while ($data = db_fetch_object($result)) {
    $rows[] = array(
      l($data->name, 'xc/search/displaying/ui/' . $data->sui_id), 
      t('!name facet group', 
        array('!name' => l($data->facet,
        'xc/search/displaying/facet/group/' . $data->facet_id))), 
      t('!name highlighter', 
        array('!name' => l($data->hlt, 
        'xc/search/displaying/mlt/' . $data->highlighter_id))), 
      l(t('edit'), 'xc/search/displaying/ui/' . $data->sui_id . '/edit'),
      l(t('delete'), 'xc/search/displaying/ui/' . $data->sui_id . '/delete'),
    );
  }
  return theme('table', $headers, $rows);
}

/**
 * Add a new server
 * @return string The add server page with a form
 */
function xc_search_ui_add() {
  return drupal_get_form('xc_search_ui_form');
}

/**
 * The form to edit search UI record
 * @return (array)
 *   The FAPI array
 */
function xc_search_ui_form() {
  $schema = drupal_get_schema_unprocessed('xc_search', 'xc_search_ui');
  drupal_set_title($schema['description']);
  $options = array(
    'omit' => array('sui_id'),
    'select' => xc_search_ui_options(),
    'label' => xc_search_ui_fieldset(),
  );
  if (empty($options['select']['facet_id'])) {
    drupal_set_message(
      t('There is no facet group defined. Please define it first!'), 'error');
  }
  $form = xc_util_build_autoform($schema, $options);
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );

  return $form;
}

/**
 * The form to modify the search UI record
 * @param $form_state (Array)
 *   The FAPI form_state
 * @param $record (Object)
 *   The search UI record
 * @return (array)
 *   The FAPI form
 */
function xc_search_ui_edit_form(&$form_state, $record) {
  $form = xc_search_ui_form();
  $form['sui_id'] = array(
    '#type'  => 'hidden',
    '#value' => $record->sui_id,
  );
  foreach (xc_search_ui_fieldset() as $name => $label) {
    $form[$name]['#default_value'] = $record->$name;
  }

  $form['submit']['#value'] = t('Save');
  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
  );

  return $form;
} 

/**
 * The handling of form submission
 * @param $form
 * @param $form_state
 * @return unknown_type
 */
function xc_search_ui_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  foreach (xc_search_ui_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }
  $ret_val = drupal_write_record('xc_search_ui', $record);
  if ($ret_val == SAVED_NEW) {
    drupal_set_message(t('Successfully added search UI settings "%name"',
      array('%name' => $record->name)));
    $form_state['redirect'] = 'xc/search/displaying/ui/' . $record->sui_id;
  }
  else {
    drupal_set_message(t('Unexpected error. Failed to create new search UI 
      settings.'));
  }
}

/**
 * Handling of modification/deletion
 * @param $form
 * @param $form_state
 * @return unknown_type
 */
function xc_search_ui_edit_form_submit($form, &$form_state) {
  $values = $form_state['clicked_button']['#post'];
  $record = new stdClass();
  $record->sui_id      = $values['sui_id'];
  $record->name        = $values['name'];

  if ($form_state['clicked_button']['#value'] == t('Delete')) {
    $sql = 'DELETE FROM {xc_search_ui} WHERE sui_id = %d';
    $result = db_query($sql, $record->sui_id);

    if ($result == SAVED_DELETED) { // repository is deleted
      drupal_set_message(t('%name removed', array('%name' => $record->name)));
      $form_state['redirect'] = 'xc/search/displaying/ui/list';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to remove %name.', 
        array('%name' => $record->name)));
    }
  }
  elseif ($form_state['clicked_button']['#value'] == t('Save')) {
    foreach (xc_search_ui_fieldset() as $name => $label) {
      $record->$name = $values[$name];
    }

    $result = drupal_write_record('xc_search_ui', $record, 'sui_id');
    if ($result == SAVED_UPDATED) {
      drupal_set_message(t('%name modified', array('%name' => $record->name)));
      $form_state['redirect'] = 'xc/search/displaying/ui/' . $record->sui_id 
        . '/view';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to modify %name.', 
        array('%name' => $record->name)));
    }
  }
  else {
    drupal_set_message(t('Unexpected error. Something else happened.'));
  }
}

/**
 * Get defaults
 * @return (String)
 *   The default values as CSV. The first line contains the field names
 */
function xc_search_ui_default() {
  xc_log_info('xc search', 'UI default');
  $raw_values = <<<END
END;

  return $raw_values;
}
