<?php
/**
 * @file
 * Functions for manipulating xc_solr_field_type table.
 * Field in the table: type_id, label, type, suffix
 */

function xc_solr_field_type_load($xc_solr_field_type) {
  $sql = "SELECT * FROM {xc_solr_field_type} WHERE type_id = %d";
  $result = db_query(db_rewrite_sql($sql, '{xc_solr_field_type}', 'type_id'), 
    $xc_solr_field_type);
  $data = db_fetch_object($result);
  return $data;
} 

function xc_solr_field_type_title($record) {
  return t('@type field type', array('@type' => $record->label));
}

function xc_solr_field_type_fieldset() {
  return array(
    'label' => t('label'), 
    'type'  => t('type'), 
    'suffix' => t('suffix'), 
  );
}

function xc_solr_field_type_options() {
  return array();
}

function xc_solr_field_type_view($record) {
  return xc_util_view($record, 
    xc_solr_field_type_fieldset()
  );
}

function xc_solr_field_type_list() {
  $output = t('List the dynamic types in Solr. The
    Drupal Toolkit will index fields accoring to field types, so in Solr an original field name
    will be stored as [fieldname]_suffix, like author_s.
    An example from a Solr\'s schema.xml: %example. Here the *_i means, that every field name,
    suffixed by "_i" will be handled as sortable integer. Consult the schema.xml file\'s comments
    for more details.
    In another page you should map the metadata schema\'s fields to one or more type,
    and the field will be indexed into those Solr field(s). 
    If you modify the schema.xml, you should keep this list syncronized, and remap the field
    names into types, otherwise the Drupal Toolkit will not know about your changes. After changes
    it is advisable reindex the whole content.', 
    array('%example' => '<dynamicField name="*_i"  type="sint" indexed="true"  stored="true"/>'));
  $headers = array(t('Label'), t('Solr base type'), t('Suffix'));
  $rows = array();
  
  $sql = "SELECT * FROM {xc_solr_field_type}";
  $result = db_query(db_rewrite_sql($sql, '{xc_solr_field_type}', 'type_id'));
  while ($data = db_fetch_object($result)) {
    $rows[] = array(
      $data->label, 
      $data->type,
      l($data->suffix, 'xc/solr/field_type/' . $data->type_id)
    );
  }
  
  $output .= theme('table', $headers, $rows);
  return $output;
}

function xc_solr_field_type_add() {
  drupal_set_title('Create a new field type');
  return drupal_get_form('xc_solr_field_type_form');
}

function xc_solr_field_type_form() {
  $schema = drupal_get_schema_unprocessed('xc_solr', 'xc_solr_field_type');
  drupal_set_title($schema['description']);
  $options = array(
    'omit' => array('type_id'),
    'hidden' => array(),
    'select' => xc_solr_field_type_options(),
    'label' => xc_solr_field_type_fieldset(),
  );
  
  $form = xc_util_build_autoform($schema, $options);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

function xc_solr_field_type_edit_form(&$form_state, $record) {
  $form = xc_solr_field_type_form($form_state, $record);
  $form['type_id'] = array(
    '#type'  => 'hidden',
    '#value' => $record->type_id,
  );
  
  foreach (xc_solr_field_type_fieldset() as $name => $label) {
    $form[$name]['#default_value'] = $record->$name;
  }
    
  $form['submit']['#value'] = t('Save');
  $form['delete'] = array(
    '#type'  => 'submit',
    '#value' => t('Delete'),
  );

  return $form;
}

function xc_solr_field_type_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  foreach (xc_solr_field_type_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }

  $ret_val = drupal_write_record('xc_solr_field_type', $record);
  if ($ret_val == SAVED_NEW) {
    drupal_set_message(t('Successfully added %type type!',
      array('%type' => $record->type)));
    $form_state['redirect'] = 'xc/solr/field_type/' . $record->type_id . '/view';
  }
  else {
    drupal_set_message(t('Unexpected error. Failed to create new Solr field type.'));
  }
}

function xc_solr_field_type_edit_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $record = new stdClass();
  $record->type_id             = $values['type_id'];
  foreach (xc_solr_field_type_fieldset() as $name => $label) {
    $record->$name = $values[$name];
  }
    
  if ($form_state['clicked_button']['#value'] == t('Delete')) {
    $sql = 'DELETE FROM {xc_solr_field_type} WHERE type_id = %d';
    $result = db_query($sql, $record->type_id);
    if ($result == SAVED_DELETED) { // repository is deleted
      drupal_set_message(t('%type removed', array('%type' => $record->type)));
      $form_state['redirect'] = 'xc/solr/field_type/list';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed to remove Solr field type.'));
    }
  }
  else {
    $ret_val = drupal_write_record('xc_solr_field_type', $record, 'type_id');
    if ($ret_val == SAVED_UPDATED) {
      drupal_set_message(t('Successfully updated "%type" type!',
        array('%type' => $record->type)));
      $form_state['redirect'] = 'xc/solr/field_type/' . $record->type_id . '/view';
    }
    else {
      drupal_set_message(t('Unexpected error. Failed modify Solr field type.'));
    }
  }
}

/**
 * Return the field type of a given label
 * @param $label (String)
 *   The label of a Solr field type
 * @return (Object)
 *   The Solr field type record (fields: type_id, label, type, suffix)
 */
function xc_solr_field_type_by_label($label) {
  $sql = "SELECT * FROM {xc_solr_field_type} WHERE label = '%s'";
  $result = db_query(db_rewrite_sql($sql, '{xc_solr_field_type}', 'type_id'), 
    $label);
  return db_fetch_object($result);
}


function xc_solr_field_type_get_all() {
  $sql = "SELECT * FROM {xc_solr_field_type}";
  $result = db_query(db_rewrite_sql($sql, '{xc_solr_field_type}', 'type_id'));
  $ret_val = array();
  while ($data = db_fetch_object($result)) {
    $ret_val[$data->suffix] = $data;
  }
  return $ret_val;
}

/**
 * Return an associative array from the suffixes and the labels of fields
 * @return assoc array The field types list
 */
function xc_solr_field_type_get_types() {
  $sql = "SELECT * FROM {xc_solr_field_type}";
  $result = db_query(db_rewrite_sql($sql, '{xc_solr_field_type}', 'type_id'));
  $ret_val = array();
  while ($data = db_fetch_object($result)) {
    $ret_val[$data->suffix] = $data->label;
  }
  return $ret_val;
}

/**
 * Create a regular expression from the actual suffixes.
 * The regex looks like this: /(_i|_s|_l|_t|_b|_f|_d|_dt|_fc|_df)$/
 * @return string The regex matching all field suffixes
 */
function xc_solr_field_type_get_regex() {
  static $suffix_regex;
  if (!isset($suffix_regex)) {
    // get suffixes
    $suffixes = array_keys(xc_solr_field_type_get_types());
    // buld regex from suffixes
    $suffix_regex = '/(' . join('|', $suffixes) . ')$/';
  }
  return $suffix_regex;
}

function xc_solr_field_type_default() {
  $defaults = <<<END
type;suffix;label
sint;_i;sortable integer
string;_s;phrase
slong;_l;sortable long
text;_t;text
boolean;_b;boolean
sfloat;_f;sortable float
sdouble;_d;sortable double
date;_dt;date
string;_fc;string facet
date;_df;date facet
END;

  return $defaults;
}
